import click
import kofnet
import tabulate


@click.command()
@click.option(
    "-i", "--input", type=click.File(), help="Path to html contents to read from."
)
def country(input: click.File):
    """Get List of countries and their corresponding codes"""
    run = kofnet.Manipulator(input.read() if input else None)
    click.secho(
        tabulate.tabulate(
            [
                [str(count), code_country[0], code_country[1]]
                for count, code_country, in enumerate(
                    run.get_countries_code_map().items(), start=1
                )
            ],
            headers=["No.", "Code", "Country"],
            tablefmt="fancy_grid",
        ),
        color="cyan",
    )


@click.command()
@click.option(
    "-i", "--input", type=click.File(), help="Path to html contents to read from."
)
@click.argument("key", required=True)
def sni(input: click.File, key: str):
    """Get SNI bug host for a particular country|code"""
    run = kofnet.Manipulator(input.read() if input else None)
    try:
        if len(key) == 2:
            click.secho(run.get_sni(key))
        else:
            code_country = run.get_countries_code_map()
            country_code = dict(
                zip(tuple(code_country.values()), tuple(code_country.keys()))
            )
            click.secho(run.get_sni(country_code[key]))
    except KeyError:
        click.secho(f"Key '{key} didn't match any country/code", fg="yellow")


@click.command()
@click.argument("output", type=click.Path(dir_okay=False, resolve_path=True))
def cache(output):
    """Download and save the html contents"""
    run = kofnet.Hunter()
    run.update_contents()
    click.secho(f"Saving contents to '{output}'")
    with open(output, "w") as fh:
        fh.write(run.html_content)


def main():
    @click.group()
    @click.version_option(kofnet.__version__, package_name="kofnet")
    def run():
        """Extract SNI bug host for different ISPs based on country"""
        pass

    run.add_command(country)
    run.add_command(sni)
    run.add_command(cache)
    try:
        run()
    except Exception as e:
        click.secho(
            f"Quitting due to {e.args[1] if e.args and len(e.args)>1 else e}", fg="red"
        )


if __name__ == "__main__":
    main()



from pathlib import Path

from setuptools import setup


INSTALL_REQUIRE = [
    "requests>=2.31.0",
]

cli_reqs = ["click==8.1.3", "tabulate==0.9.0"]


EXTRA_REQUIRE = {
    "cli": cli_reqs,
    "all": INSTALL_REQUIRE + cli_reqs,
}

DOCS_PATH = Path(__file__).parents[0] / "docs/README.md"
PATH = Path("README.md")
if not PATH.exists():
    with Path.open(DOCS_PATH, encoding="utf-8") as f1:
        with Path.open(PATH, "w+", encoding="utf-8") as f2:
            f2.write(f1.read())

setup(
    name="kofnet",
    version="0.0.2",
    license="MIT",
    author="Smartwa",
    maintainer="Smartwa",
    author_email="simatwacaleb@proton.me",
    description="Extract SNI bug host for different ISPs based on country",
    packages=["kofnet"],
    url="https://github.com/Simatwa/kofnet",
    project_urls={
        "Bug Report": "https://github.com/Simatwa/kofnet/issues/new",
        "Homepage": "https://github.com/Simatwa/kofnet",
        "Source Code": "https://github.com/Simatwa/kofnet",
        "Issue Tracker": "https://github.com/Simatwa/kofnet/issues",
        "Download": "https://github.com/Simatwa/kofnet/releases",
        "Documentation": "https://github.com/Simatwa/kofnet/blob/main/docs",
    },
    entry_points={
        "console_scripts": [
            "kofnet = kofnet.console:main",
        ],
    },
    install_requires=INSTALL_REQUIRE,
    extras_require=EXTRA_REQUIRE,
    python_requires=">=3.10",
    keywords=["kofnet"],
    long_description=Path.open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: Free For Home Use",
        "Intended Audience :: Customer Service",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)



import re
import requests

from importlib import metadata

try:
    __version__ = metadata.version("kofnet")
except metadata.PackageNotFoundError:
    __version__ = "1.0.0"

__repo__ = "https://github.com/Simatwa/kofnet"

__info__ = "Extract SNI bug host for different ISPs based on country"

__author__ = "Smartwa"

__all__ = ["Hunter", "Manipulator"]


class Hunter:
    """Hunt SNI bugs for different countries"""

    url = "https://kofnet.co.za/sni-bug-host-generator/"

    def __init__(self, contents: str = None, url: str = ""):
        """Constructor

        Args:
            contents (str, optional): Html Contents. Defaults to None.
            url (str, optional): url to replace the generator link at the extracted SNIs. Defaults to "".
        """
        self.html_content = contents
        self.url = url
        self.update_contents()

    def update_contents(self, ignore_check: bool = False) -> str:
        """Fetch new html contents

        Args:
            ignore_check (bool, optional): Ignore cache check instead fetch new content. Defaults to False.

        Returns:
            str: html contents
        """
        if not ignore_check and self.html_content is not None:
            return self.html_content
        else:
            resp = requests.get(self.url, timeout=20)
            resp.raise_for_status()
            self.html_content = resp.text
            return resp.text

    def get_countries_code_map(self) -> dict[str, str]:
        """Extract codes and their corresponding countries.

        Returns:
            dict[str, str]: code : country
        """
        code_countries: str = re.findall(
            r"var\sisoCountries=(\{[^{}]*\})", self.html_content
        )[0]
        code_countries_2: list[tuple[str, str]] = re.findall(
            r'([A-Z]{2}):"(\w*|[\w\s]*)"',
            code_countries,
        )
        cache = {}
        for code, country in code_countries_2:
            cache[code] = country

        return cache

    def get_code_sni_map(self) -> dict[str, str]:
        """Extract sni bug host mapped with their country codes.

        Returns:
            dict[str, str]: {code : sni}
        """
        code_sni_map = r"data\s*=\s*\{[^{}]*\}"
        hunt = re.findall(code_sni_map, self.html_content)
        target: str = hunt[0]

        results = re.findall(r"\W*([A-Z]{2}):\W*`([\w\W][^`]*)\W*", target)

        cache = {}
        for code, sni in results:
            sanitized_sni = re.sub(r"</?a[^>]*>", self.url, sni)
            cache[code] = sanitized_sni

        return cache


class Manipulator(Hunter):
    """Manipulate country, code and sni of `Hunter` class"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cached = {
            "country_code": self.get_countries_code_map(),
            "code_sni": self.get_code_sni_map(),
        }

    def update_cache(self):
        """Fetch new html contents and update cache"""
        self.update_contents(ignore_check=True)
        self.cached["country_code"] = self.get_countries_code_map()
        self.cached["code_sni"] = self.get_code_sni_map()

    def get_country(self, code: str, default: str = None) -> str | None:
        """Convert country code to country name

        Args:
            code (str): Country abbreviation.
            default (str): Default counrry name to be returned. Default to None

        Returns:
            str|None: Country name.
        """
        return self.cached["country_code"].get(code, default)

    def get_sni(self, code: str) -> str:
        """Get SNI for a particular  country by code

        Args:
            code (str): Country code

        Returns:
            str|None: SNI bug
        """
        return self.cached["code_sni"][code]


if __name__ == "__main__":
    run = Manipulator(open("assets/kofnet-cache1.html", "r").read())
    a = run.get_code_sni_map()
    while True:
        try:
            print(run.get_sni(input(">>")))
        except KeyError:
            print("No Match!")
